------------------------------------------
-- Dynamic include library for Euphoria --
--  written by Gabriel Boehme 5/7/1999  --
--    legal status updated 3/28/2000    --
------------------------------------------

-- Distribution Status
-- -------------------
-- This software is now being contributed to the Public Domain.
-- Feel free to use it, abuse it, and/or modify it for any purpose.
-- Send bug reports or feature requests to:
--      gabrielboehme@hotmail.com
--
--
-- Disclaimer
-- ----------
-- I am in no way responsible for any damage, data loss, or any other
-- adverse effects resulting from the use/abuse/etc. of this library.
--
--
-- Acknowledgments
-- ---------------
-- BIG thanks to Mark Honnor of Liquid-Nitrogen Software for the original idea!
--
-- Thanks also to Robert Craig for the beauty of Euphoria.
include misc.e

type valid_include_name(sequence s)
   -- programmer-defined format for temporary include file names
   if length(s) and equal(s[1], '_') then
      -- verify that the include file name begins with an underscore
      return 1
   end if
   --if length(s) >= 5 and equal(s[length(s)-3..length(s)], ".tmp") then
   --   -- verify that the include file name has a .tmp extension
   --   return 1
   --end if
   printf(2, "ERROR: include file name %s is not in the correct format\n", {s})
   return 0
end type

sequence InclList, InclStatus
InclList = {}
InclStatus = {}

constant DELETED = 0

global procedure create_include(valid_include_name incl, sequence list)
integer fn
sequence name
   -- verify list
   for i = 1 to length(list) do
      if atom(list[i]) then
	 puts(2, "ERROR: List must contain sequences, not atoms\n")
	 ? 1/0
      end if
      name = list[i]
      for j = 1 to length(name) do
	 if sequence(name[j]) then
	    puts(2, "ERROR: File names must not contain sequences\n")
	    ? 1/0
	 end if
      end for
   end for
   
   -- verify include file name
   if find(incl, InclList) then
      printf(2, "ERROR: %s has already been included\n", {incl})
      ? 1/0
   end if
   InclList = append(InclList, incl)
   InclStatus = append(InclStatus, not DELETED)
   
   fn = open(incl, "w")
   if fn = -1 then
      printf(2, "ERROR: Cannot create include file %s\n", {incl})
      ? 1/0
   end if
   
   -- write out file, close and exit
   for i = 1 to length(list) do
      printf(fn, "include %s\n", {list[i]}) 
   end for
   close(fn)
end procedure

global procedure delete_include(sequence incl)
integer x
   x = find(incl, InclList)
   if x then
      if InclStatus[x] = DELETED then
	 printf(2, "ERROR: %s has already been deleted\n", {incl})
	 ? 1/0
      else
	 InclStatus[x] = DELETED
	 if platform() = LINUX then
	    system("rm -f " & incl, 2)
	 else
	    system("del " & incl, 2)
	 end if
      end if
   else
      printf(2, "ERROR: %s has not been created as a temp include file\n", {incl})
      ? 1/0
   end if
end procedure

