-- EuOS Filesystem Routines
-- by Jeffrey Fielding
-- JJProg@cyberbury.net
-- File Descriptor Section
-- 1.4: added some error checking and handling
-- 1.3: BUGFIXED - k_translatefd was using the wrong index into the task data
include sort.e
constant K_FS_CLOSE = 1,
	 K_FS_READ = 2,
	 K_FS_WRITE = 3,
	 K_FS_SEEK = 4,
	 K_FS_TELL = 5,
	 K_FS_IOCTL = 6
sequence k_fds, k_fd_handlers
k_fds = {-2,-2}
k_fd_handlers = {}
function k_verifyfd(integer fd)
	if fd > 0 and fd <= length(k_fds) then
		if k_fds[fd] != -1 then
			return 1
		end if
	end if
	k_setLastError(1,"Invalid fd.")
	return 0
end function
function k_verifyfdhandler(integer h)
	if h > 0 and h <= length(k_fd_handlers) then
		return 1
	end if
	k_setLastError(2,"Invalid fd handler.")
	return 0
end function
global function k_allocfd(integer handler)
	integer i
	if not k_verifyfdhandler(handler) then
		return 0
	end if
	i = find(-1,k_fds)
	if i then
		k_fds[i] = handler
		return i
	else
		k_fds &= handler
		return length(k_fds)
	end if
end function
global procedure k_freefd(integer fd)
	if k_verifyfd(fd) then
		k_fds[fd] = -1
	end if
end procedure
function k_translatefd(integer fd)
	if fd >= 0 and fd <= 2 then
		return k_getTaskData(k_getPid(),3+fd)
	else
		return fd
	end if
end function
global function k_close(integer fd)
	fd = k_translatefd(fd)
	if not k_verifyfd(fd) then
		return -1
	end if
	return call_func(k_fd_handlers[k_fds[fd]][K_FS_CLOSE],{fd})
end function
global function k_read(integer fd, integer n)
	fd = k_translatefd(fd)
	if not k_verifyfd(fd) then
		return -1
	end if
	return call_func(k_fd_handlers[k_fds[fd]][K_FS_READ],{fd,n})
end function
global function k_write(integer fd, object data)
	fd = k_translatefd(fd)
	if not k_verifyfd(fd) then
		return -1
	end if
	return call_func(k_fd_handlers[k_fds[fd]][K_FS_WRITE],{fd,data})
end function
global function k_seek(integer fd, atom where)
	fd = k_translatefd(fd)
	if not k_verifyfd(fd) then
		return -1
	end if
	return call_func(k_fd_handlers[k_fds[fd]][K_FS_SEEK],{fd,where})
end function
global function k_tell(integer fd)
	fd = k_translatefd(fd)
	if not k_verifyfd(fd) then
		return -1
	end if
	return call_func(k_fd_handlers[k_fds[fd]][K_FS_TELL],{fd})
end function
global function k_ioctl(integer fd, integer op, object arg)
	fd = k_translatefd(fd)
	if not k_verifyfd(fd) then
		return -1
	end if
	return call_func(k_fd_handlers[k_fds[fd]][K_FS_IOCTL],{fd,op,arg})
end function
global function k_new_fd_handler(sequence handlers)
	k_fd_handlers = append(k_fd_handlers,handlers)
	return length(k_fd_handlers)
end function
constant K_FS_OPEN = 1,
	 K_FS_DIR = 2,
	 K_FS_DEL = 3,
	 K_FS_MKDIR = 4,
	 K_FS_ONMOUNT = 5,
	 K_FS_ONUNMOUNT = 6
sequence k_fs_roots, k_fs_types
sequence k_fs_type_names, k_fs_type_routines
k_fs_roots = {}
k_fs_types = {}
k_fs_type_names = {}
k_fs_type_routines = {}
global procedure k_register_fs_type(sequence name, sequence routines)
	k_fs_type_names = append(k_fs_type_names,name)
	k_fs_type_routines = append(k_fs_type_routines,routines)
end procedure
function k_getroot(sequence x)
	for i = 1 to length(k_fs_roots) do
		if match(k_fs_roots[i],x) = 1 then
			return i
		end if
	end for
	return 0
end function
global function k_open(sequence file, sequence mode)
	integer root
	root = k_getroot(file)
	if root then
		return call_func(
			k_fs_type_routines[k_fs_types[root]][K_FS_OPEN],
			{file,mode})
	else
		return -1
	end if
end function
global function k_dir(sequence file)
	integer root
	root = k_getroot(file)
	if root then
		return call_func(
			k_fs_type_routines[k_fs_types[root]][K_FS_DIR],
			{file})
	else
		return -1
	end if
end function
global function k_mkdir(sequence file)
	integer root
	root = k_getroot(file)
	if root then
		return call_func(
			k_fs_type_routines[k_fs_types[root]][K_FS_MKDIR],
			{file})
	else
		return -1
	end if
end function
global function k_del(sequence file)
	integer root
	root = k_getroot(file)
	if root then
		return call_func(
			k_fs_type_routines[k_fs_types[root]][K_FS_DEL],
			{file})
	else
		return -1
	end if
end function
function k_counttree(sequence p)
	integer n
	n = 0
	for i = 1 to length(p) do
		n += (p[i] = '/')
	end for
	return n
end function
procedure k_insertroot(sequence root, integer typ)
	integer insertpoint, n
	n = k_counttree(root)
	insertpoint = -1
	for i = 1 to length(k_fs_roots) do
		if k_counttree(k_fs_roots[i]) < n then
			insertpoint = i
			exit
		end if
	end for
	if insertpoint = -1 then
		k_fs_roots = append(k_fs_roots,root)
		k_fs_types &= typ
	else
		k_fs_roots = append(k_fs_roots[1..insertpoint-1],root) &
			k_fs_roots[insertpoint..length(k_fs_roots)]
		k_fs_types = k_fs_types[1..insertpoint-1] & typ &
			k_fs_types[insertpoint..length(k_fs_types)]
	end if
end procedure
global procedure k_mount(sequence dev, sequence root, sequence typ)
	integer j
	if root[length(root)] != '/' then
		root &= '/'
	end if
	j = find(typ,k_fs_type_names)
	if j then
		k_insertroot(root,j)
		call_proc(k_fs_type_routines[j][K_FS_ONMOUNT],{dev, root})
	end if
end procedure
global procedure k_umount(sequence root)
	integer i
	i = find(root,k_fs_roots)
	if i then
		call_proc(k_fs_type_routines[i][K_FS_ONUNMOUNT],{root})
		k_fs_roots[i] = -1
	end if
end procedure
global procedure k_unmountAll()
	for i = 1 to length(k_fs_roots) do
		if sequence(k_fs_roots[i]) then
			k_umount(k_fs_roots[i])
		end if
	end for
end procedure









